;
; Mr Window Manager v2
; Author:			Desi Quintans <me@desiquintans.com>
; Website:			http://www.desiquintans.com
;
; Script Function:
;		Allows you to assign a window to a Numpad key, and then automatically minimise/restore the window
;		by pressing that key while Num Lock is off. The Numpad can still be used normally while Num Lock
;		is on (it is on by default). Press NumpadDel to list all of the existing key assigments, and press
;		Win+NumpadDel thrice in a row to delete all of the assignments.
;

#NoEnv
#Persistent
#WinActivateForce ; Suppresses any taskbar flashing that may happen.
SendMode Input
SetWorkingDir %A_ScriptDir%

; Initialise the Window-storing variables.
	window_1 = 0
	window_2 = 0
	window_3 = 0
	window_4 = 0
	window_5 = 0
	window_6 = 0
	window_7 = 0
	window_8 = 0
	window_9 = 0
	window_0 = 0


; This function is called when Win+NumpadKey is pressed. It takes the ID of the active window and
; stores it in the window variable given in the function parameter.
	WinKeyManager(ByRef window_var)
	{
		window_var := WinExist("A")
	}

; This function is called when a NumpadKey is pressed by itself. It checks if the window stored in the
; variable given in the function parameter exists, and maximises/restores it if it does. If non-
; existent, it clears the variable.
	KeyManager(ByRef window_var)
	{
		if (window_var != 0)
		{
			if WinExist("ahk_id" . window_var) ; If variable points to existing window
			{
				WinGet, is_minimised, MinMax, ahk_id %window_var%
				if (is_minimised == -1) ; Window is minimised
				{
					WinActivate, ahk_id %window_var%
				}
				else
				{
					WinMinimize, ahk_id %window_var% ; Minimise restored window.
				}
				return
			}

			; Since the script didn't Return, the variable points to a window that does not exist.
			window_var = 0 ; Clear the variable.
		}
	}

; This function retrieves the Window Title linked to the window ID passed in the parameter.
GetTitle(ByRef window_var)
{
	WinGetTitle, window_title, ahk_id %window_var%
	StringLeft, truncated_title, window_title, 23
	return %truncated_title%
}

; All the code hereafter remaps the Numpad and Win+Numpad keys to their new Mr Window Manager roles.

; If user mashes Win+NumpadDel rapidly three times in a row, all window assignments
; are wiped from memory.
#NumpadDel::
	If (A_TickCount-lasttime < 400)
	{
		If (A_TickCount-lasttime2 < 400)
		{
			window_1 = 0
			window_2 = 0
			window_3 = 0
			window_4 = 0
			window_5 = 0
			window_6 = 0
			window_7 = 0
			window_8 = 0
			window_9 = 0
			window_0 = 0
			delete_message = "All those keypresses! Amazing! The colors! `nI've, umm, forgotten all of the windows now."
			TrayTip, Mr Window Manager says, %delete_message%, 11, 1
		}
		lasttime2 := A_TickCount
	}
	lasttime := A_TickCount
return

; Pressing NumpadDel displays a list of what windows Mr Window Manager is remembering on which keys.
NumpadDel::
	w0 := GetTitle(window_0)
	w1 := GetTitle(window_1)
	w2 := GetTitle(window_2)
	w3 := GetTitle(window_3)
	w4 := GetTitle(window_4)
	w5 := GetTitle(window_5)
	w6 := GetTitle(window_6)
	w7 := GetTitle(window_7)
	w8 := GetTitle(window_8)
	w9 := GetTitle(window_9)
	
	window_listing = 0: %w0% `n1: %w1% `n2: %w2% `n3: %w3% `n4: %w4% `n5: %w5% `n6: %w6% `n7: %w7% `n8: %w8% `n9: %w9%
	TrayTip, Mr Window Manager is remembering:, %window_listing%, 30, 1
return

#NumpadIns::
	WinKeyManager(window_0)
return

NumpadIns::
	KeyManager(window_0)
return

#NumpadEnd::
	WinKeyManager(window_1)
return

NumpadEnd::
	KeyManager(window_1)
return

#NumpadDown::
	WinKeyManager(window_2)
return

NumpadDown::
	KeyManager(window_2)
return

#NumpadPgDn::
	WinKeyManager(window_3)
return

NumpadPgDn::
	KeyManager(window_3)
return

#NumpadLeft::
	WinKeyManager(window_4)
return

NumpadLeft::
	KeyManager(window_4)
return

#NumpadClear::
	WinKeyManager(window_5)
return

NumpadClear::
	KeyManager(window_5)
return

#NumpadRight::
	WinKeyManager(window_6)
return

NumpadRight::
	KeyManager(window_6)
return

#NumpadHome::
	WinKeyManager(window_7)
return

NumpadHome::
	KeyManager(window_7)
return

#NumpadUp::
	WinKeyManager(window_8)
return

NumpadUp::
	KeyManager(window_8)
return

#NumpadPgUp::
	WinKeyManager(window_9)
return

NumpadPgUp::
	KeyManager(window_9)
return